""" event used by the handler. """

import threading

from .command import Command

def __dir__():
    return ("Event",)

class Event(Command):

    """ Basic event class. """

    def __init__(self):
        super().__init__()
        self._ready = threading.Event()
        self.batch = False
        self.direct = False
        self.type = "chat"
        self.name = ""

    def prompt(self):
        """ optional prompt. """

    def ready(self):
        self._ready.set()

    def reply(self, txt):
        """ reply to the origin. """
        from ob.kernel import k
        self.result.append(txt)
        if self.direct:
            bot = k.fleet.get_bot(self.orig)
            bot.say(self.channel, txt, self.type)
        elif not self.batch:
            k.fleet.say(self.orig, self.channel, txt, self.type)

    def wait(self):
        """ wait for event to finish. """
        self._ready.wait()
        thrs = []
        vals = []
        for thr in self._thrs:
            try:
                thr.join()
            except RuntimeError:
                vals.append(thr)
                continue
            thrs.append(thr)
        for val in vals:
            try:
                val.join()
            except RuntimeError:
                pass
        for thr in thrs:
            self._thrs.remove(thr)
        return self
